@@ -31,6 +31,9 @@
 
 #include <string.h>
 
+#include "hd-desktop.h"
+#include "hd-display.h"
+
 #include "hd-status-area-box.h"
 #include "hd-status-menu.h"
 #include "hd-status-menu-config.h"
@@ -67,6 +70,10 @@ struct _HDStatusAreaPrivate
 {
   HDPluginManager *plugin_manager;
 
+  HDDesktop *desktop;
+  HDDisplay *display;
+  GList *status_plugins;
+
   GtkWidget *status_menu;
 
   GtkWidget *icon_box;
@@ -75,7 +82,10 @@ struct _HDStatusAreaPrivate
 
   GtkWidget *clock_box;
 
-  gboolean resize_after_map;
+  GtkWidget *main_alignment;
+
+  gboolean resize_after_map : 1;
+  gboolean status_area_visible;
 };
 
 G_DEFINE_TYPE (HDStatusArea, hd_status_area, GTK_TYPE_WINDOW);
@@ -92,16 +102,80 @@ button_press_event_cb (GtkWidget      *widget,
   return TRUE;
 }
 
+static gboolean
+is_widget_on_screen (GtkWidget *widget)
+{
+  GdkWindow *window;
+  gint x, y, width, height;
+
+  window = gtk_widget_get_window (widget);
+  gdk_window_get_root_origin (window, &x, &y);
+  gdk_window_get_geometry (window, NULL, NULL, &width, &height, NULL);
+
+  /* the compositor moves obscured windows off the screen, so we can use
+   * that to determine whether the status area is visible */
+  return (x + width > 0) && (y + height > 0);
+}
+
+static void
+update_status_area_visibility (HDStatusArea *status_area)
+{
+  HDStatusAreaPrivate *priv = status_area->priv;
+  gboolean visible;
+  GList *l;
+
+  visible = (is_widget_on_screen (GTK_WIDGET (status_area)) &&
+             !hd_desktop_is_task_switcher_visible (priv->desktop) &&
+             hd_display_is_on (priv->display));
+
+  if (visible != priv->status_area_visible)
+    {
+      priv->status_area_visible = visible;
+
+      /* inform status area plugins if the status area is obscured or not */
+      for (l = priv->status_plugins; l; l = l->next)
+        {
+          g_object_set (l->data, "status-area-visible", visible, NULL);
+        }
+    }
+}
+
+static gboolean
+configure_event_cb (GtkWidget         *widget,
+                    GdkEventConfigure *event,
+                    gpointer           user_data)
+{
+  HDStatusArea *status_area;
+
+  status_area = HD_STATUS_AREA (widget);
+
+  update_status_area_visibility (status_area);
+
+  return FALSE;
+}
+
 static void
 hd_status_area_init (HDStatusArea *status_area)
 {
   HDStatusAreaPrivate *priv = HD_STATUS_AREA_GET_PRIVATE (status_area);
-  GtkWidget *main_alignment, *left_alignment, *main_hbox, *left_hbox, *special_hbox;
+  GtkWidget *left_alignment, *main_hbox, *left_hbox, *special_hbox;
   guint i;
 
   /* Set priv member */
   status_area->priv = priv;
 
+  priv->desktop = hd_desktop_get ();
+  g_signal_connect_swapped (priv->desktop, "task-switcher-show",
+                            G_CALLBACK (update_status_area_visibility), status_area);
+  g_signal_connect_swapped (priv->desktop, "task-switcher-hide",
+                            G_CALLBACK (update_status_area_visibility), status_area);
+  priv->display = hd_display_get ();
+  g_signal_connect_swapped (priv->display, "display-status-changed",
+                            G_CALLBACK (update_status_area_visibility), status_area);
+  update_status_area_visibility (status_area);
+
+  priv->status_plugins = NULL;
+
   /* Create Status area UI */
   gtk_widget_add_events (GTK_WIDGET (status_area), GDK_BUTTON_PRESS_MASK);
   g_signal_connect (G_OBJECT (status_area), "button-press-event",
@@ -109,11 +183,8 @@ hd_status_area_init (HDStatusArea *status_area)
   gtk_widget_set_app_paintable (GTK_WIDGET (status_area), TRUE);
   gtk_widget_set_size_request (GTK_WIDGET (status_area), -1, STATUS_AREA_HEIGHT);
 
-  main_alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-  gtk_alignment_set_padding (GTK_ALIGNMENT (main_alignment),
-                             CUSTOM_MARGIN_9, CUSTOM_MARGIN_9,
-                             HILDON_MARGIN_DOUBLE, HILDON_MARGIN_DOUBLE);
-  gtk_widget_show (main_alignment);
+  priv->main_alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+  gtk_widget_show (priv->main_alignment);
 
   left_alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
   gtk_alignment_set_padding (GTK_ALIGNMENT (left_alignment),
@@ -147,8 +218,8 @@ hd_status_area_init (HDStatusArea *status_area)
   gtk_widget_show (priv->icon_box);
 
   /* Pack widgets */
-  gtk_container_add (GTK_CONTAINER (status_area), main_alignment);
-  gtk_container_add (GTK_CONTAINER (main_alignment), main_hbox);
+  gtk_container_add (GTK_CONTAINER (status_area), priv->main_alignment);
+  gtk_container_add (GTK_CONTAINER (priv->main_alignment), main_hbox);
   gtk_box_pack_start (GTK_BOX (main_hbox), left_alignment, FALSE, FALSE, 0);
   gtk_container_add (GTK_CONTAINER (left_alignment), left_hbox);
   gtk_box_pack_start (GTK_BOX (left_hbox), priv->clock_box, FALSE, FALSE, 0);
@@ -156,6 +227,11 @@ hd_status_area_init (HDStatusArea *status_area)
   for (i = 0; i < HD_STATUS_AREA_NUM_SPECIAL_ITEMS; i++)
     gtk_box_pack_start (GTK_BOX (special_hbox), priv->special_item_image[i], FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (main_hbox), priv->icon_box, TRUE, TRUE, 0);
+
+  /* Detect when the entire status area is moved off screen (this happens when a
+   * program is full-screen) */
+  g_signal_connect (G_OBJECT (status_area), "configure-event",
+                    G_CALLBACK (configure_event_cb), status_area);
 }
 
 static GObject *
@@ -180,11 +256,28 @@ hd_status_area_constructor (GType                  type,
 static void
 hd_status_area_dispose (GObject *object)
 {
-  HDStatusAreaPrivate *priv = HD_STATUS_AREA (object)->priv;
+  HDStatusArea *status_area = HD_STATUS_AREA (object);
+  HDStatusAreaPrivate *priv = status_area->priv;
 
   if (priv->plugin_manager)
     priv->plugin_manager = (g_object_unref (priv->plugin_manager), NULL);
 
+  if (priv->desktop)
+    {
+      g_signal_handlers_disconnect_by_func (priv->desktop,
+                                            update_status_area_visibility,
+                                            status_area);
+      priv->desktop = (g_object_unref (priv->desktop), NULL);
+    }
+
+  if (priv->display)
+    {
+      g_signal_handlers_disconnect_by_func (priv->display,
+                                            update_status_area_visibility,
+                                            status_area);
+      priv->display = (g_object_unref (priv->display), NULL);
+    }
+
   G_OBJECT_CLASS (hd_status_area_parent_class)->dispose (object);
 }
 
@@ -324,6 +417,9 @@ hd_status_area_plugin_added_cb (HDPluginManager *plugin_manager,
                                position);
     }
 
+  priv->status_plugins = g_list_prepend (priv->status_plugins, plugin);
+  g_object_set (plugin, "status-area-visible", priv->status_area_visible, NULL);
+
   g_signal_connect (plugin, "notify::status-area-icon",
                     G_CALLBACK (status_area_icon_changed), NULL);
   status_area_icon_changed (HD_STATUS_PLUGIN_ITEM (plugin));
@@ -367,6 +463,7 @@ hd_status_area_plugin_removed_cb (HDPluginManager *plugin_manager,
                              priv->clock_box);
     }
 
+  priv->status_plugins = g_list_remove (priv->status_plugins, plugin);
   g_object_unref (plugin);
 }
 
@@ -462,6 +559,33 @@ hd_status_area_expose_event (GtkWidget *widget,
                                                                        event);
 }
 
+static gboolean
+is_portrait_mode (GtkWidget *widget)
+{
+  GdkScreen *screen;
+
+  screen = gtk_widget_get_screen (widget);
+
+  return gdk_screen_get_height (screen) > gdk_screen_get_width (screen);
+}
+
+static void
+update_alignemnt_padding (HDStatusArea *status_area)
+{
+  HDStatusAreaPrivate *priv = status_area->priv;
+  gboolean portrait = is_portrait_mode (GTK_WIDGET (status_area));
+  guint left_right_padding;
+
+  if (portrait)
+    left_right_padding = HILDON_MARGIN_DEFAULT;
+  else
+    left_right_padding = HILDON_MARGIN_DOUBLE;
+
+  gtk_alignment_set_padding (GTK_ALIGNMENT (priv->main_alignment),
+                             CUSTOM_MARGIN_9, CUSTOM_MARGIN_9,
+                             left_right_padding, left_right_padding);
+}
+
 static void
 hd_status_area_realize (GtkWidget *widget)
 {
@@ -475,6 +599,11 @@ hd_status_area_realize (GtkWidget *widget)
   gtk_widget_set_colormap (widget,
                            gdk_screen_get_rgba_colormap (screen));
 
+  g_signal_connect_swapped (screen, "size-changed",
+                            G_CALLBACK (update_alignemnt_padding),
+                            widget);
+  update_alignemnt_padding (HD_STATUS_AREA (widget));
+
   gtk_widget_set_app_paintable (widget,
                                 TRUE);
 
@@ -506,6 +635,20 @@ hd_status_area_realize (GtkWidget *widget)
   gdk_window_set_back_pixmap (widget->window, pixmap, FALSE);
 }
 
+static void
+hd_status_area_unrealize (GtkWidget *widget)
+{
+  GdkScreen *screen;
+
+  screen = gtk_widget_get_screen (widget);
+  g_signal_handlers_disconnect_by_func (screen,
+                                        update_alignemnt_padding,
+                                        HD_STATUS_AREA (widget));
+
+  GTK_WIDGET_CLASS (hd_status_area_parent_class)->unrealize (widget);
+}
+
+
 static void
 hd_status_area_map (GtkWidget *widget)
 {
@@ -570,11 +713,11 @@ hd_status_area_check_resize (GtkContainer *container)
            * the required size (will result in a configure notify event
            * see above) */
           gdk_window_resize (widget->window, req.width, req.height);
-
-          /* Resize children (also if size not changed and so no
-            configure notify event is triggered) */
-          gtk_container_resize_children (GTK_CONTAINER (widget));
         }
+
+      /* Resize children (also if size not changed and so no
+         configure notify event is triggered) */
+      gtk_container_resize_children (GTK_CONTAINER (widget));
     }
 }
 
@@ -594,6 +737,7 @@ hd_status_area_class_init (HDStatusAreaClass *klass)
   object_class->set_property = hd_status_area_set_property;
 
   widget_class->realize = hd_status_area_realize;
+  widget_class->unrealize = hd_status_area_unrealize;
   widget_class->map = hd_status_area_map;
   widget_class->expose_event = hd_status_area_expose_event;
 
